@@ -52,10 +52,9 @@ static const char * const cmd_device_add_usage[] = {
 	"btrfs device add [options] <device> [<device>...] <path>",
 	"Add one or more devices to a mounted filesystem.",
 	"",
-	"-K|--nodiscard    do not perform whole device TRIM on devices that report such capability",
-	"-f|--force        force overwrite existing filesystem on the disk",
-	"--enqueue         wait if there's another exclusive operation running,",
-	"                  otherwise continue",
+	OPTLINE("-K|--nodiscard", "do not perform whole device TRIM on devices that report such capability"),
+	OPTLINE("-f|--force", "force overwrite existing filesystem on the disk"),
+	OPTLINE("--enqueue", "wait if there's another exclusive operation running, otherwise continue"),
 	NULL
 };
 
@@ -335,8 +334,7 @@ static const char * const cmd_device_remove_usage[] = {
 	"Remove a device from a filesystem",
 	COMMON_USAGE_REMOVE_DELETE,
 	"",
-	"--enqueue         wait if there's another exclusive operation running,",
-	"                  otherwise continue",
+	OPTLINE("--enqueue", "wait if there's another exclusive operation running, otherwise continue"),
 	NULL
 };
 
@@ -352,8 +350,7 @@ static const char * const cmd_device_delete_usage[] = {
 	"Remove a device from a filesystem (alias of \"btrfs device remove\")",
 	COMMON_USAGE_REMOVE_DELETE,
 	"",
-	"--enqueue         wait if there's another exclusive operation running,",
-	"                  otherwise continue",
+	OPTLINE("--enqueue", "wait if there's another exclusive operation running, otherwise continue"),
 	NULL
 };
 
@@ -397,9 +394,8 @@ static const char * const cmd_device_scan_usage[] = {
 	"The reverse is done by the forget option, such devices must be unmounted.",
 	"No argument will unregister all devices that are not part of a mounted filesystem.",
 	"",
-	" -d|--all-devices            enumerate and register all devices, use as a fallback",
-	"                             if blkid is not available",
-	" -u|--forget [<device>...]   unregister a given device or all stale devices if no path ",
+	OPTLINE("-d|--all-devices", "enumerate and register all devices, use as a fallback if blkid is not available"),
+	OPTLINE("-u|--forget [<device>...]", "unregister a given device or all stale devices if no path"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
 	HELPINFO_INSERT_QUIET,
@@ -440,10 +436,10 @@ static int cmd_device_scan(const struct cmd_struct *cmd, int argc, char **argv)
 	devstart = optind;
 
 	if (all && forget)
-		usage(cmd);
+		usage(cmd, 1);
 
 	if (all && check_argc_max(argc - optind, 1))
-		usage(cmd);
+		usage(cmd, 1);
 
 	if (all || argc - optind == 0) {
 		if (forget) {
@@ -571,9 +567,9 @@ static const char * const cmd_device_stats_usage[] = {
 	"Show device IO error statistics for all devices of the given filesystem",
 	"identified by PATH or DEVICE. The filesystem must be mounted.",
 	"",
-	"-c|--check             return non-zero if any stat counter is not zero",
-	"-z|--reset             show current stats and reset values to zero",
-	"-T                     show current stats in tabular format",
+	OPTLINE("-c|--check", "return non-zero if any stat counter is not zero"),
+	OPTLINE("-z|--reset", "show current stats and reset values to zero"),
+	OPTLINE("-T", "show current stats in tabular format"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_FORMAT,
 	NULL
@@ -652,16 +648,6 @@ static int print_device_stat_tabular(struct string_table *table, int row,
 	char *canonical_path = path_canonicalize(path);
 	int j;
 	int err = 0;
-	static const struct {
-		const char name[32];
-		enum btrfs_dev_stat_values stat_idx;
-	} dev_stats[] = {
-		{ "write_io_errs", BTRFS_DEV_STAT_WRITE_ERRS },
-		{ "read_io_errs", BTRFS_DEV_STAT_READ_ERRS },
-		{ "flush_io_errs", BTRFS_DEV_STAT_FLUSH_ERRS },
-		{ "corruption_errs", BTRFS_DEV_STAT_CORRUPTION_ERRS },
-		{ "generation_errs", BTRFS_DEV_STAT_GENERATION_ERRS },
-	};
 
 	/* Skip header + --- line */
 	row += 2;
@@ -681,20 +667,14 @@ static int print_device_stat_tabular(struct string_table *table, int row,
 	table_printf(table, 1, row, ">%s", canonical_path);
 	free(canonical_path);
 
-	for (j = 0; j < ARRAY_SIZE(dev_stats); j++) {
-		enum btrfs_dev_stat_values stat_idx = dev_stats[j].stat_idx;
+	table_printf(table, 2, row, ">%llu", args->values[BTRFS_DEV_STAT_WRITE_ERRS]);
+	table_printf(table, 3, row, ">%llu", args->values[BTRFS_DEV_STAT_READ_ERRS]);
+	table_printf(table, 4, row, ">%llu", args->values[BTRFS_DEV_STAT_FLUSH_ERRS]);
+	table_printf(table, 5, row, ">%llu", args->values[BTRFS_DEV_STAT_CORRUPTION_ERRS]);
+	table_printf(table, 6, row, ">%llu", args->values[BTRFS_DEV_STAT_GENERATION_ERRS]);
 
-		/* We got fewer items than we know */
-		if (args->nr_items < stat_idx + 1)
-			continue;
-
-		table_printf(table, 2, row, ">%llu", args->values[stat_idx]);
-		table_printf(table, 3, row, ">%llu", args->values[stat_idx]);
-		table_printf(table, 4, row, ">%llu", args->values[stat_idx]);
-		table_printf(table, 5, row, ">%llu", args->values[stat_idx]);
-		table_printf(table, 6, row, ">%llu", args->values[stat_idx]);
-
-		if (check && (args->values[stat_idx] > 0))
+	for (j = 0; j < BTRFS_DEV_STAT_VALUES_MAX; j++) {
+		if (check && (args->values[j] > 0))
 			err |= 64;
 	}
 
