@@ -191,6 +191,10 @@ failure_recovery() {
 
 	run_check $SUDO_HELPER udevadm settle
 
+	# Scan to make sure btrfs detects both devices before trying to mount
+	run_check $SUDO_HELPER "$TOP/btrfs" device scan "$loop1"
+	run_check $SUDO_HELPER "$TOP/btrfs" device scan "$loop2"
+
 	# Mount and unmount, on trans commit all disks should be consistent
 	run_check $SUDO_HELPER mount "$loop1" "$TEST_MNT"
 	run_check $SUDO_HELPER umount "$TEST_MNT"
